import os
import curses
import curses.textpad
from pathlib import Path

def get_directory_contents(path, file_types, selection):
   try:
       return sorted(
           [item for item in Path(path).iterdir() 
            if item.is_dir() or item.suffix in file_types or str(item) in selection],
           key=lambda x: (x.is_file(), x.name.lower())
       )
   except PermissionError:
       return []

def mark_subitems(selection, ignored, base_path, state, file_types):
   for item in Path(base_path).rglob('*'):
       path = str(item)
       if state and path in ignored:
           ignored.pop(path)
           
       if state:
           if item.is_dir() or item.suffix in file_types or path in selection:
               selection[path] = True
       else:
           selection.pop(path, None)

def mark_ignored(ignored, selection, base_path):
   for item in Path(base_path).rglob('*'):
       path = str(item)
       if path in selection:
           selection.pop(path)
       ignored[path] = True
   ignored[str(base_path)] = True

def curses_file_selector(stdscr, start_path=".", preselected_filters=None, included_files=None, ignored_dirs=None, included_dirs=None):
    curses.curs_set(0)
    stdscr.clear()
    
    current_path = Path(start_path).resolve()
    selection = {}
    ignored = {}
    global file_types
    file_types = set(preselected_filters) if preselected_filters else set()

  # Auto-select current directory contents if no specific paths
    if start_path == "." and not (included_files or included_dirs):
        for item in current_path.iterdir():
            if item.is_dir() or (file_types and item.suffix in file_types):
                path = str(item.resolve())
                selection[path] = True
                if item.is_dir():
                    mark_subitems(selection, ignored, path, True, file_types)
                    
    # Preload included files and directories
    if included_files:
        for file_path in included_files:
            path = str(Path(file_path).resolve())
            selection[path] = True

    if included_dirs:
        for dir_path in included_dirs:
            path = str(Path(dir_path).resolve())
            mark_subitems(selection, ignored, path, True, file_types)
            selection[path] = True

    # Preload ignored directories
    if ignored_dirs:
        for dir_path in ignored_dirs:
            path = str(Path(dir_path).resolve())
            mark_ignored(ignored, selection, path)

    stack = [current_path]
    idx = 0
    top_line = 0

    while True:
        stdscr.clear()
        height, width = stdscr.getmaxyx()

        # Show full path
        path_str = str(current_path)
        if len(path_str) >= width:
            path_str = "..." + path_str[-(width-3):]
        stdscr.addstr(0, 0, path_str, curses.A_BOLD)
        stdscr.addstr(1, 0, "─" * min(len(path_str), width-1))

        # Help text
        stdscr.addstr(2, 0, "↑/↓:Navigate  SPACE:Toggle  ←/→:Navigate dirs  i:Ignore  v:View  f:Filter  q:Done", curses.A_DIM)
        
        items = get_directory_contents(current_path, file_types, selection)
        display_items = ["[../] (Up one level)"] + [
            f"[{'✔' if selection.get(str(item), False) else '✘' if ignored.get(str(item), False) else ' '}] {item.name}/" if item.is_dir() else
            f"[{'✔' if selection.get(str(item), False) else '✘' if ignored.get(str(item), False) else ' '}] {item.name}"
            for item in items
        ]
        
        if idx >= len(display_items):
            idx = max(len(display_items) - 1, 0)

        if idx < top_line:
            top_line = idx
        elif idx >= top_line + height - 4:
            top_line = idx - (height - 5)

        for i, line in enumerate(display_items[top_line:top_line + height - 4]):
            truncated_line = (line[:width - 3] + "...") if len(line) >= width else line
            if i + top_line == idx:
                stdscr.addstr(i + 3, 0, truncated_line, curses.A_REVERSE)
            else:
                stdscr.addstr(i + 3, 0, truncated_line)
        
        key = stdscr.getch()
        
        if key == curses.KEY_DOWN and idx < len(display_items) - 1:
            idx += 1
        elif key == curses.KEY_UP and idx > 0:
            idx -= 1
        elif key in [curses.KEY_RIGHT, 10]:  # Right arrow or Enter
            if idx > 0:
                selected_item = items[idx - 1]
                if selected_item.is_dir():
                    stack.append(selected_item)
                    current_path = selected_item
                    idx = 0
                    top_line = 0
        elif key in [curses.KEY_LEFT, 127, 8]:  # Left arrow or Backspace
            if len(stack) > 1:
                stack.pop()
                current_path = stack[-1]
                idx = 0
                top_line = 0
        elif key == ord(' '):
            if idx > 0:
                selected_item = items[idx - 1]
                path = str(selected_item)
                
                if path in ignored:
                    for key in list(ignored.keys()):
                        if key.startswith(path):
                            ignored.pop(key)
                    state = True
                    selection[path] = True
                else:
                    state = not selection.get(path, False)
                    if not state:
                        selection.pop(path, None)
                    else:
                        selection[path] = True
                        
                if selected_item.is_dir():
                    mark_subitems(selection, ignored, selected_item, state, file_types)

        elif key == ord('i') and idx > 0:
            selected_item = items[idx - 1]
            path = str(selected_item)
            
            if path in selection:
                for key in list(selection.keys()):
                    if key.startswith(path):
                        selection.pop(key)
                mark_ignored(ignored, selection, selected_item)
            else:
                if path in ignored:
                    for key in list(ignored.keys()):
                        if key.startswith(path):
                            ignored.pop(key)
                else:
                    mark_ignored(ignored, selection, selected_item)

        elif key == ord('f'):
            curses.curs_set(1)
            stdscr.clear()
            stdscr.addstr(0, 0, f"Edit file extensions (comma-separated, no dots needed, current: {', '.join(file_types)}): ")

            edit_win = curses.newwin(1, 50, 1, 0)
            edit_win.addstr(",".join(ft.lstrip('.') for ft in file_types))
            edit_box = curses.textpad.Textbox(edit_win, insert_mode=True)

            stdscr.refresh()
            new_input = edit_box.edit().strip()

            if new_input:
                file_types = {f".{ext.strip()}" if not ext.strip().startswith(".") else ext.strip() 
                            for ext in new_input.split(',')}
            curses.curs_set(0)

        elif key == ord('v'):
            def view_selections():
                v_idx = 0
                v_top = 0
                selected_paths = list(selection.keys())
                ignored_paths = list(ignored.keys())
                all_paths = selected_paths + ignored_paths
                
                while True:
                    stdscr.clear()
                    stdscr.addstr(0, 0, "Selected & Ignored Files/Directories:", curses.A_BOLD)
                    stdscr.addstr(1, 0, "✔: Selected  ✘: Ignored", curses.A_DIM)
                    
                    for i, path in enumerate(all_paths[v_top:v_top + height - 4]):
                        prefix = "✔ " if path in selected_paths else "✘ "
                        if i + v_top == v_idx:
                            stdscr.addstr(i + 2, 0, prefix + path[:width - 3], curses.A_REVERSE)
                        else:
                            stdscr.addstr(i + 2, 0, prefix + path[:width - 3])
                            
                    stdscr.addstr(height - 1, 0, "Use UP/DOWN to scroll, 'q' to exit")
                    key_v = stdscr.getch()
                    
                    if key_v == curses.KEY_DOWN and v_idx < len(all_paths) - 1:
                        v_idx += 1
                        if v_idx >= v_top + height - 4:
                            v_top += 1
                    elif key_v == curses.KEY_UP and v_idx > 0:
                        v_idx -= 1
                        if v_idx < v_top:
                            v_top -= 1
                    elif key_v == ord('q'):
                        break
            view_selections()

        elif key == 27:  # Escape
            return None
        elif key == ord('q'):
            return {
                "selected": [path for path, checked in selection.items() if checked],
                "ignored": [path for path, is_ignored in ignored.items() if is_ignored],
                "file_types": list(file_types)
            }

def select_files(start_path=".", preselected_filters=None, included_files=None, ignored_dirs=None, included_dirs=None):
    return curses.wrapper(curses_file_selector, 
                         start_path=start_path,
                         preselected_filters=preselected_filters,
                         included_files=included_files,
                         ignored_dirs=ignored_dirs,
                         included_dirs=included_dirs)